package org.example.lombok;

import lombok.extern.java.Log;
import org.junit.Test;

import java.util.Date;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

@Log
public class BookTest {

    private final static String AUTHOR = "Robert C. Martin";
    private final static String TITLE  = "Clean Code";
    private final static String ISBN   = "978-0132350884";
    private final static Date RELEASE_DATE = new Date(1218405600000L); // August 11, 2008
    private final static Integer PAGES = 464;
    private final static String TO_STRING = "Book{author=Robert C. Martin, title=Clean Code, isbn=978-0132350884, releaseDate=Mon Aug 11 00:00:00 CEST 2008, pages=464}";
    private final static String TO_STRING2 = "Book(author=Robert C. Martin, title=Clean Code, isbn=978-0132350884, releaseDate=Mon Aug 11 00:00:00 CEST 2008, pages=464)";
    private final static String TO_STRING3 = "Book{author='Robert C. Martin', title='Clean Code', isbn='978-0132350884', releaseDate=Mon Aug 11 00:00:00 CEST 2008, pages=464}";

    @Test
    public void shouldUseEmptyConstructorAndPropertiesToConstructBook() {
        //given
        Book book = new Book();
        book.setAuthor(AUTHOR);
        book.setTitle(TITLE);
        book.setIsbn(ISBN);
        book.setReleaseDate(RELEASE_DATE);
        book.setPages(PAGES);

        //expect
        assertBookPropertiesEqualStaticFields(book);
    }

    @Test
    public void shouldUseFullConstructorAndPropertiesToConstructBook() {
        //given
        Book book = createBookWithFullConstructor();

        //expect
        assertBookPropertiesEqualStaticFields(book);
    }

    @Test
    public void shouldHaveFullToString() {
        //given
        Book book = createBookWithFullConstructor();

        //expect
        assertEquals(TO_STRING2, book.toString());
    }

    @Test
    public void shouldImplementEqualsProperly() {
        //expect
        assertTrue(createBookWithFullConstructor().equals(createBookWithFullConstructor()));
    }

    @Test
    public void shouldImplementHashCodeProperly() {
        //expect
        assertEquals(createBookWithFullConstructor().hashCode(), createBookWithFullConstructor().hashCode());
    }

    private Book createBookWithFullConstructor() {
        return new Book(AUTHOR, TITLE, ISBN, RELEASE_DATE, PAGES);
    }

    private void assertBookPropertiesEqualStaticFields(Book book) {
        assertEquals(AUTHOR, book.getAuthor());
        assertEquals(TITLE, book.getTitle());
        assertEquals(ISBN, book.getIsbn());
        assertEquals(RELEASE_DATE, book.getReleaseDate());
        assertEquals(PAGES, book.getPages());
    }
}
